
# q-expansion of newform 435.3.h.b, downloaded from the LMFDB on 28 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 435
weight = 3
poly_data = [5, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[146, [-1, 0]], [262, [1, 0]], [31, [-1, 0]]]
aps_data = [[2, 0], [2, -1], [0, 1], [8, 0], [11, 0], [2, 0], [32, 0], [0, -6], [0, -15], [-29, 0], [0, 12], [0, 15], [-37, 0], [0, 21], [2, 0], [0, 9], [0, 30], [0, -6], [38, 0], [0, -42], [0, 33], [0, -6], [0, 21], [-166, 0], [0, 63], [11, 0], [26, 0], [0, -78], [101, 0], [-142, 0], [0, 63], [86, 0], [-94, 0], [-19, 0], [0, 90], [-193, 0], [0, 54], [0, -111], [0, 126], [0, -99], [0, -96], [-61, 0], [23, 0], [0, 54], [0, -105], [101, 0], [0, 126], [2, 0], [0, -33], [0, -90], [0, -183], [0, 84], [59, 0], [26, 0], [0, -57], [284, 0], [74, 0], [0, -6], [248, 0], [0, -42], [-424, 0], [278, 0], [0, -105], [431, 0], [-268, 0], [-208, 0], [0, -48], [0, -126], [0, -201], [653, 0], [0, -138], [317, 0], [0, -9], [656, 0], [0, 210], [0, 321], [-31, 0], [-646, 0], [0, 300], [0, -282], [0, -126], [0, 294], [0, 114], [0, 189], [26, 0], [338, 0], [509, 0], [68, 0], [383, 0], [-208, 0], [-124, 0], [-598, 0], [-466, 0], [278, 0], [-814, 0], [698, 0], [0, -162], [0, 156], [152, 0], [0, 414], [128, 0], [0, -33], [-892, 0], [-166, 0], [-613, 0], [0, 222], [0, 210], [0, -330], [914, 0], [0, -252], [0, 162], [332, 0], [-604, 0], [0, -240], [-1006, 0], [-409, 0], [-208, 0], [0, -369], [-922, 0], [-1171, 0], [1319, 0], [212, 0], [1262, 0], [0, 9], [614, 0], [0, -588], [-247, 0], [0, -414], [0, 42], [0, -426], [0, -72], [278, 0], [0, 246], [0, -273], [0, 90], [0, -252], [74, 0], [-46, 0], [26, 0], [1229, 0], [779, 0], [0, -372], [0, 222], [-1024, 0], [0, 180], [194, 0], [0, -651], [0, -705], [0, -636], [0, 510], [-856, 0], [-1129, 0], [-124, 0], [632, 0], [0, -177], [971, 0], [-814, 0], [0, -288], [-1342, 0], [0, 576], [-1678, 0], [0, 294], [0, 483], [443, 0], [0, 51], [-346, 0], [1319, 0], [0, 327]]
